@@ -1,8 +1,7 @@
 import numpy as np
 
-from ..color import rgb2gray
 from ..util.dtype import dtype_range, dtype_limits
-from .._shared.utils import warn
+from .._shared import utils
 
 
 __all__ = ['histogram', 'cumulative_distribution', 'equalize_hist',
@@ -14,7 +13,7 @@ DTYPE_RANGE.update((d.__name__, limits) for d, limits in dtype_range.items())
 DTYPE_RANGE.update({'uint10': (0, 2 ** 10 - 1),
                     'uint12': (0, 2 ** 12 - 1),
                     'uint14': (0, 2 ** 14 - 1),
-                    'bool': dtype_range[np.bool_],
+                    'bool': dtype_range[bool],
                     'float': dtype_range[np.float64]})
 
 
@@ -35,7 +34,22 @@ def _offset_array(arr, low_boundary, high_boundary):
     return arr, offset
 
 
-def _bincount_histogram(image, source_range):
+def _bincount_histogram_centers(image, source_range):
+    """Compute bin centers for bincount-based histogram."""
+    if source_range not in ['image', 'dtype']:
+        raise ValueError(
+            f'Incorrect value for `source_range` argument: {source_range}'
+        )
+    if source_range == 'image':
+        image_min = int(image.min().astype(np.int64))
+        image_max = int(image.max().astype(np.int64))
+    elif source_range == 'dtype':
+        image_min, image_max = dtype_limits(image, clip_negative=False)
+    bin_centers = np.arange(image_min, image_max + 1)
+    return bin_centers
+
+
+def _bincount_histogram(image, source_range, bin_centers=None):
     """
     Efficient histogram calculation for an image of integers.
 
@@ -58,32 +72,130 @@ def _bincount_histogram(image, source_range):
     bin_centers : array
         The values at the center of the bins.
     """
-    if source_range not in ['image', 'dtype']:
-        raise ValueError('Incorrect value for `source_range` argument: {}'.format(source_range))
-    if source_range == 'image':
-        image_min = np.min(image).astype(np.int64)
-        image_max = np.max(image).astype(np.int64)
-    elif source_range == 'dtype':
-        image_min, image_max = dtype_limits(image, clip_negative=False)
+    if bin_centers is None:
+        bin_centers = _bincount_histogram_centers(image, source_range)
+    image_min, image_max = bin_centers[0], bin_centers[-1]
     image, offset = _offset_array(image, image_min, image_max)
     hist = np.bincount(image.ravel(), minlength=image_max - image_min + 1)
-    bin_centers = np.arange(image_min, image_max + 1)
     if source_range == 'image':
         idx = max(image_min, 0)
         hist = hist[idx:]
     return hist, bin_centers
 
 
-def histogram(image, nbins=256, source_range='image', normalize=False):
+def _get_outer_edges(image, hist_range):
+    """Determine the outer bin edges to use for `numpy.histogram`.
+
+    These are obtained from either the image or hist_range.
+
+    Parameters
+    ----------
+    image : ndarray
+        Image for which the histogram is to be computed.
+    hist_range: 2-tuple of int or None
+        Range of values covered by the histogram bins. If None, the minimum
+        and maximum values of `image` are used.
+
+    Returns
+    -------
+    first_edge, last_edge : int
+        The range spanned by the histogram bins.
+
+    Notes
+    -----
+    This function is adapted from ``np.lib.histograms._get_outer_edges``.
+    """
+    if hist_range is not None:
+        first_edge, last_edge = hist_range
+        if first_edge > last_edge:
+            raise ValueError(
+                "max must be larger than min in hist_range parameter."
+            )
+        if not (np.isfinite(first_edge) and np.isfinite(last_edge)):
+            raise ValueError(
+                f'supplied hist_range of [{first_edge}, {last_edge}] is '
+                f'not finite'
+            )
+    elif image.size == 0:
+        # handle empty arrays. Can't determine hist_range, so use 0-1.
+        first_edge, last_edge = 0, 1
+    else:
+        first_edge, last_edge = image.min(), image.max()
+        if not (np.isfinite(first_edge) and np.isfinite(last_edge)):
+            raise ValueError(
+                f'autodetected hist_range of [{first_edge}, {last_edge}] is '
+                f'not finite'
+            )
+
+    # expand empty hist_range to avoid divide by zero
+    if first_edge == last_edge:
+        first_edge = first_edge - 0.5
+        last_edge = last_edge + 0.5
+
+    return first_edge, last_edge
+
+
+def _get_bin_edges(image, nbins, hist_range):
+    """Computes histogram bins for use with `numpy.histogram`.
+
+    Parameters
+    ----------
+    image : ndarray
+        Image for which the histogram is to be computed.
+    nbins : int
+        The number of bins.
+    hist_range: 2-tuple of int
+        Range of values covered by the histogram bins.
+
+    Returns
+    -------
+    bin_edges : ndarray
+        The histogram bin edges.
+
+    Notes
+    -----
+    This function is a simplified version of
+    ``np.lib.histograms._get_bin_edges`` that only supports uniform bins.
+    """
+    first_edge, last_edge = _get_outer_edges(image, hist_range)
+    # numpy/gh-10322 means that type resolution rules are dependent on array
+    # shapes. To avoid this causing problems, we pick a type now and stick
+    # with it throughout.
+    bin_type = np.result_type(first_edge, last_edge, image)
+    if np.issubdtype(bin_type, np.integer):
+        bin_type = np.result_type(bin_type, float)
+
+    # compute bin edges
+    bin_edges = np.linspace(
+        first_edge, last_edge, nbins + 1, endpoint=True, dtype=bin_type
+    )
+    return bin_edges
+
+
+def _get_numpy_hist_range(image, source_range):
+    if source_range == 'image':
+        hist_range = None
+    elif source_range == 'dtype':
+        hist_range = dtype_limits(image, clip_negative=False)
+    else:
+        ValueError('Wrong value for the `source_range` argument')
+    return hist_range
+
+
+@utils.channel_as_last_axis(multichannel_output=False)
+def histogram(image, nbins=256, source_range='image', normalize=False, *,
+              channel_axis=None):
     """Return histogram of image.
 
     Unlike `numpy.histogram`, this function returns the centers of bins and
     does not rebin integer arrays. For integer arrays, each integer value has
     its own bin, which improves speed and intensity-resolution.
 
-    The histogram is computed on the flattened image: for color images, the
-    function should be used separately on each channel to obtain a histogram
-    for each color channel.
+    If `channel_axis` is not set, the histogram is computed on the flattened
+    image. For color or multichannel images, set ``channel_axis`` to use a
+    common binning for all channels. Alternatively, one may apply the function
+    separately on each channel to obtain a histogram for each color channel
+    with separate binning.
 
     Parameters
     ----------
@@ -98,11 +210,16 @@ def histogram(image, nbins=256, source_range='image', normalize=False):
         of that data type.
     normalize : bool, optional
         If True, normalize the histogram by the sum of its values.
+    channel_axis : int or None, optional
+        If None, the image is assumed to be a grayscale (single channel) image.
+        Otherwise, this parameter indicates which axis of the array corresponds
+        to channels.
 
     Returns
     -------
     hist : array
-        The values of the histogram.
+        The values of the histogram. When ``channel_axis`` is not None, hist
+        will be a 2D array where the first axis corresponds to channels.
     bin_centers : array
         The values at the center of the bins.
 
@@ -115,28 +232,72 @@ def histogram(image, nbins=256, source_range='image', normalize=False):
     >>> from skimage import data, exposure, img_as_float
     >>> image = img_as_float(data.camera())
     >>> np.histogram(image, bins=2)
-    (array([107432, 154712]), array([ 0. ,  0.5,  1. ]))
+    (array([ 93585, 168559]), array([0. , 0.5, 1. ]))
     >>> exposure.histogram(image, nbins=2)
-    (array([107432, 154712]), array([ 0.25,  0.75]))
+    (array([ 93585, 168559]), array([0.25, 0.75]))
     """
     sh = image.shape
-    if len(sh) == 3 and sh[-1] < 4:
-        warn("This might be a color image. The histogram will be "
-             "computed on the flattened image. You can instead "
-             "apply this function to each color channel.")
+    if len(sh) == 3 and sh[-1] < 4 and channel_axis is None:
+        utils.warn('This might be a color image. The histogram will be '
+                   'computed on the flattened image. You can instead '
+                   'apply this function to each color channel, or set '
+                   'channel_axis.')
+
+    if channel_axis is not None:
+        channels = sh[-1]
+        hist = []
+
+        # compute bins based on the raveled array
+        if np.issubdtype(image.dtype, np.integer):
+            # here bins corresponds to the bin centers
+            bins = _bincount_histogram_centers(image, source_range)
+        else:
+            # determine the bin edges for np.histogram
+            hist_range = _get_numpy_hist_range(image, source_range)
+            bins = _get_bin_edges(image, nbins, hist_range)
+
+        for chan in range(channels):
+            h, bc = _histogram(image[..., chan], bins, source_range, normalize)
+            hist.append(h)
+        # Convert to numpy arrays
+        bin_centers = np.asarray(bc)
+        hist = np.stack(hist, axis=0)
+    else:
+        hist, bin_centers = _histogram(image, nbins, source_range, normalize)
+
+    return hist, bin_centers
+
+
+def _histogram(image, bins, source_range, normalize):
+    """
+
+    Parameters
+    ----------
+    image : ndarray
+        Image for which the histogram is to be computed.
+    bins : int or ndarray
+        The number of histogram bins. For images with integer dtype, an array
+        containing the bin centers can also be provided. For images with
+        floating point dtype, this can be an array of bin_edges for use by
+        ``np.histogram``.
+    source_range : string, optional
+        'image' (default) determines the range from the input image.
+        'dtype' determines the range from the expected range of the images
+        of that data type.
+    normalize : bool, optional
+        If True, normalize the histogram by the sum of its values.
+    """
 
     image = image.flatten()
     # For integer types, histogramming with bincount is more efficient.
     if np.issubdtype(image.dtype, np.integer):
-        hist, bin_centers = _bincount_histogram(image, source_range)
+        bin_centers = bins if isinstance(bins, np.ndarray) else None
+        hist, bin_centers = _bincount_histogram(
+            image, source_range, bin_centers
+        )
     else:
-        if source_range == 'image':
-            hist_range = None
-        elif source_range == 'dtype':
-            hist_range = dtype_limits(image, clip_negative=False)
-        else:
-            ValueError('Wrong value for the `source_range` argument')
-        hist, bin_edges = np.histogram(image, bins=nbins, range=hist_range)
+        hist_range = _get_numpy_hist_range(image, source_range)
+        hist, bin_edges = np.histogram(image, bins=bins, range=hist_range)
         bin_centers = (bin_edges[:-1] + bin_edges[1:]) / 2.
 
     if normalize:
@@ -181,6 +342,11 @@ def cumulative_distribution(image, nbins=256):
     hist, bin_centers = histogram(image, nbins)
     img_cdf = hist.cumsum()
     img_cdf = img_cdf / float(img_cdf[-1])
+
+    # cast img_cdf to single precision for float32 or float16 inputs
+    cdf_dtype = utils._supported_float_type(image.dtype)
+    img_cdf = img_cdf.astype(cdf_dtype, copy=False)
+
     return img_cdf, bin_centers
 
 
@@ -195,7 +361,7 @@ def equalize_hist(image, nbins=256, mask=None):
         Number of bins for image histogram. Note: this argument is
         ignored for integer images, for which each integer is its own
         bin.
-    mask: ndarray of bools or 0s and 1s, optional
+    mask : ndarray of bools or 0s and 1s, optional
         Array of same shape as `image`. Only points at which mask == True
         are used for the equalization, which is applied to the whole image.
 
@@ -220,7 +386,10 @@ def equalize_hist(image, nbins=256, mask=None):
     else:
         cdf, bin_centers = cumulative_distribution(image, nbins)
     out = np.interp(image.flat, bin_centers, cdf)
-    return out.reshape(image.shape)
+    out = out.reshape(image.shape)
+    # Unfortunately, np.interp currently always promotes to float64, so we
+    # have to cast back to single precision when float32 output is desired
+    return out.astype(utils._supported_float_type(image.dtype), copy=False)
 
 
 def intensity_range(image, range_values='image', clip_negative=False):
@@ -266,6 +435,53 @@ def intensity_range(image, range_values='image', clip_negative=False):
     return i_min, i_max
 
 
+def _output_dtype(dtype_or_range, image_dtype):
+    """Determine the output dtype for rescale_intensity.
+
+    The dtype is determined according to the following rules:
+    - if ``dtype_or_range`` is a dtype, that is the output dtype.
+    - if ``dtype_or_range`` is a dtype string, that is the dtype used, unless
+      it is not a NumPy data type (e.g. 'uint12' for 12-bit unsigned integers),
+      in which case the data type that can contain it will be used
+      (e.g. uint16 in this case).
+    - if ``dtype_or_range`` is a pair of values, the output data type will be
+      ``_supported_float_type(image_dtype)``. This preserves float32 output for
+      float32 inputs.
+
+    Parameters
+    ----------
+    dtype_or_range : type, string, or 2-tuple of int/float
+        The desired range for the output, expressed as either a NumPy dtype or
+        as a (min, max) pair of numbers.
+    image_dtype : np.dtype
+        The input image dtype.
+
+    Returns
+    -------
+    out_dtype : type
+        The data type appropriate for the desired output.
+    """
+    if type(dtype_or_range) in [list, tuple, np.ndarray]:
+        # pair of values: always return float.
+        return utils._supported_float_type(image_dtype)
+    if type(dtype_or_range) == type:
+        # already a type: return it
+        return dtype_or_range
+    if dtype_or_range in DTYPE_RANGE:
+        # string key in DTYPE_RANGE dictionary
+        try:
+            # if it's a canonical numpy dtype, convert
+            return np.dtype(dtype_or_range).type
+        except TypeError:  # uint10, uint12, uint14
+            # otherwise, return uint16
+            return np.uint16
+    else:
+        raise ValueError(
+            'Incorrect value for out_range, should be a valid image data '
+            f'type or a pair of values, got {dtype_or_range}.'
+        )
+
+
 def rescale_intensity(image, in_range='image', out_range='dtype'):
     """Return image after stretching or shrinking its intensity levels.
 
@@ -297,6 +513,12 @@ def rescale_intensity(image, in_range='image', out_range='dtype'):
         Image array after rescaling its intensity. This image is the same dtype
         as the input image.
 
+    Notes
+    -----
+    .. versionchanged:: 0.17
+        The dtype of the output array has changed to match the input dtype, or
+        float if the output range is specified by a pair of floats.
+
     See Also
     --------
     equalize_hist
@@ -314,42 +536,68 @@ def rescale_intensity(image, in_range='image', out_range='dtype'):
     It's easy to accidentally convert an image dtype from uint8 to float:
 
     >>> 1.0 * image
-    array([  51.,  102.,  153.])
+    array([ 51., 102., 153.])
 
     Use `rescale_intensity` to rescale to the proper range for float dtypes:
 
     >>> image_float = 1.0 * image
     >>> rescale_intensity(image_float)
-    array([ 0. ,  0.5,  1. ])
+    array([0. , 0.5, 1. ])
 
     To maintain the low contrast of the original, use the `in_range` parameter:
 
     >>> rescale_intensity(image_float, in_range=(0, 255))
-    array([ 0.2,  0.4,  0.6])
+    array([0.2, 0.4, 0.6])
 
     If the min/max value of `in_range` is more/less than the min/max image
     intensity, then the intensity levels are clipped:
 
     >>> rescale_intensity(image_float, in_range=(0, 102))
-    array([ 0.5,  1. ,  1. ])
+    array([0.5, 1. , 1. ])
 
     If you have an image with signed integers but want to rescale the image to
-    just the positive range, use the `out_range` parameter:
+    just the positive range, use the `out_range` parameter. In that case, the
+    output dtype will be float:
 
     >>> image = np.array([-10, 0, 10], dtype=np.int8)
     >>> rescale_intensity(image, out_range=(0, 127))
+    array([  0. ,  63.5, 127. ])
+
+    To get the desired range with a specific dtype, use ``.astype()``:
+
+    >>> rescale_intensity(image, out_range=(0, 127)).astype(np.int8)
     array([  0,  63, 127], dtype=int8)
 
+    If the input image is constant, the output will be clipped directly to the
+    output range:
+    >>> image = np.array([130, 130, 130], dtype=np.int32)
+    >>> rescale_intensity(image, out_range=(0, 127)).astype(np.int32)
+    array([127, 127, 127], dtype=int32)
     """
-    dtype = image.dtype.type
+    if out_range in ['dtype', 'image']:
+        out_dtype = _output_dtype(image.dtype.type, image.dtype)
+    else:
+        out_dtype = _output_dtype(out_range, image.dtype)
 
-    imin, imax = intensity_range(image, in_range)
-    omin, omax = intensity_range(image, out_range, clip_negative=(imin >= 0))
+    imin, imax = map(float, intensity_range(image, in_range))
+    omin, omax = map(float, intensity_range(image, out_range,
+                                            clip_negative=(imin >= 0)))
+
+    if np.any(np.isnan([imin, imax, omin, omax])):
+        utils.warn(
+            "One or more intensity levels are NaN. Rescaling will broadcast "
+            "NaN to the full image. Provide intensity levels yourself to "
+            "avoid this. E.g. with np.nanmin(image), np.nanmax(image).",
+            stacklevel=2
+        )
 
     image = np.clip(image, imin, imax)
 
-    image = (image - imin) / float(imax - imin)
-    return np.array(image * (omax - omin) + omin, dtype=dtype)
+    if imin != imax:
+        image = (image - imin) / (imax - imin)
+        return np.asarray(image * (omax - omin) + omin, dtype=out_dtype)
+    else:
+        return np.clip(image, omin, omax).astype(out_dtype)
 
 
 def _assert_non_negative(image):
@@ -360,6 +608,15 @@ def _assert_non_negative(image):
                          'skimage.exposure.rescale_intensity.')
 
 
+def _adjust_gamma_u8(image, gamma, gain):
+    """LUT based implementation of gamma adjustment.
+
+    """
+    lut = (255 * gain * (np.linspace(0, 1, 256) ** gamma))
+    lut = np.minimum(lut, 255).astype('uint8')
+    return lut[image]
+
+
 def adjust_gamma(image, gamma=1, gain=1):
     """Performs Gamma Correction on the input image.
 
@@ -406,24 +663,31 @@ def adjust_gamma(image, gamma=1, gain=1):
     >>> image.mean() > gamma_corrected.mean()
     True
     """
-    _assert_non_negative(image)
-    dtype = image.dtype.type
-
     if gamma < 0:
         raise ValueError("Gamma should be a non-negative real number.")
 
-    scale = float(dtype_limits(image, True)[1] - dtype_limits(image, True)[0])
+    dtype = image.dtype.type
 
-    out = ((image / scale) ** gamma) * scale * gain
-    return dtype(out)
+    if dtype is np.uint8:
+        out = _adjust_gamma_u8(image, gamma, gain)
+    else:
+        _assert_non_negative(image)
+
+        scale = float(dtype_limits(image, True)[1]
+                      - dtype_limits(image, True)[0])
+
+        out = (((image / scale) ** gamma) * scale * gain).astype(dtype)
+
+    return out
 
 
 def adjust_log(image, gain=1, inv=False):
     """Performs Logarithmic correction on the input image.
 
     This function transforms the input image pixelwise according to the
-    equation ``O = gain*log(1 + I)`` after scaling each pixel to the range 0 to 1.
-    For inverse logarithmic correction, the equation is ``O = gain*(2**I - 1)``.
+    equation ``O = gain*log(1 + I)`` after scaling each pixel to the range
+    0 to 1. For inverse logarithmic correction, the equation is
+    ``O = gain*(2**I - 1)``.
 
     Parameters
     ----------
@@ -458,7 +722,7 @@ def adjust_log(image, gain=1, inv=False):
         return dtype(out)
 
     out = np.log2(1 + image / scale) * scale * gain
-    return dtype(out)
+    return out.astype(dtype)
 
 
 def adjust_sigmoid(image, cutoff=0.5, gain=10, inv=False):
@@ -495,7 +759,7 @@ def adjust_sigmoid(image, cutoff=0.5, gain=10, inv=False):
     ----------
     .. [1] Gustav J. Braun, "Image Lightness Rescaling Using Sigmoidal Contrast
            Enhancement Functions",
-           http://www.cis.rit.edu/fairchild/PDFs/PAP07.pdf
+           http://markfairchild.org/PDFs/PAP07.pdf
 
     """
     _assert_non_negative(image)
@@ -507,7 +771,7 @@ def adjust_sigmoid(image, cutoff=0.5, gain=10, inv=False):
         return dtype(out)
 
     out = (1 / (1 + np.exp(gain * (cutoff - image / scale)))) * scale
-    return dtype(out)
+    return out.astype(dtype)
 
 
 def is_low_contrast(image, fraction_threshold=0.05, lower_percentile=1,
@@ -535,6 +799,11 @@ def is_low_contrast(image, fraction_threshold=0.05, lower_percentile=1,
     out : bool
         True when the image is determined to be low contrast.
 
+    Notes
+    -----
+    For boolean images, this function returns False only if all values are
+    the same (the method, threshold, and percentile arguments are ignored).
+
     References
     ----------
     .. [1] https://scikit-image.org/docs/dev/user_guide/data_types.html
@@ -551,8 +820,17 @@ def is_low_contrast(image, fraction_threshold=0.05, lower_percentile=1,
     False
     """
     image = np.asanyarray(image)
-    if image.ndim == 3 and image.shape[2] in [3, 4]:
-        image = rgb2gray(image)
+
+    if image.dtype == bool:
+        return not ((image.max() == 1) and (image.min() == 0))
+
+    if image.ndim == 3:
+        from ..color import rgb2gray, rgba2rgb  # avoid circular import
+
+        if image.shape[2] == 4:
+            image = rgba2rgb(image)
+        if image.shape[2] == 3:
+            image = rgb2gray(image)
 
     dlimits = dtype_limits(image, clip_negative=False)
     limits = np.percentile(image, [lower_percentile, upper_percentile])
